package com.example.playertime;

import com.mojang.brigadier.arguments.IntegerArgumentType;
import net.fabricmc.api.ModInitializer;
import net.fabricmc.fabric.api.command.v2.CommandRegistrationCallback;
import net.fabricmc.fabric.api.event.lifecycle.v1.ServerLifecycleEvents;
import net.fabricmc.fabric.api.networking.v1.ServerPlayConnectionEvents;
import net.fabricmc.loader.api.FabricLoader;
import net.minecraft.server.command.CommandManager;
import net.minecraft.server.command.ServerCommandSource;
import net.minecraft.server.network.ServerPlayerEntity;
import net.minecraft.text.ClickEvent;
import net.minecraft.text.MutableText;
import net.minecraft.text.Text;
import net.minecraft.util.Util;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.stream.Collectors;

public class PlayerTimeMod implements ModInitializer {
    public static final Logger LOGGER = LoggerFactory.getLogger("PlayerTimeTracker");
    private static PlayerTimeTracker timeTracker;
    private static WebServer webServer;
    private static ModConfig config;

    @Override
    public void onInitialize() {

        config = new ModConfig(FabricLoader.getInstance().getConfigDir());


        try {
            LOGGER.info("[在线时间] 初始化玩家在线时长视奸MOD");


            ServerLifecycleEvents.SERVER_STARTING.register(server -> {
                timeTracker = new PlayerTimeTracker(server);
                try {
                    webServer = new WebServer(timeTracker, config.getWebPort(), server); // 传入 MinecraftServer
                    webServer.start();
                    LOGGER.info("[在线时间] Web服务器在端口 " + config.getWebPort() + " 启动");
                } catch (Exception e) {
                    LOGGER.error("[在线时间] 无法启动Web服务器", e);
                }
            });

            ServerPlayConnectionEvents.JOIN.register((handler, sender, server) -> {
                if (timeTracker != null) {
                    timeTracker.onPlayerJoin(handler.player);
                }
            });

            ServerPlayConnectionEvents.DISCONNECT.register((handler, server) -> {
                if (timeTracker != null) {
                    timeTracker.onPlayerLeave(handler.player);
                }
            });

            ServerLifecycleEvents.SERVER_STOPPING.register(server -> {
                LOGGER.info("[在线时间] 服务器停止 - 保存数据");
                if (webServer != null) {
                    webServer.stop();
                }
                if (timeTracker != null) {
                    timeTracker.saveAll();
                }
            });

        } catch (Exception e) {
            LOGGER.error("[在线时间] Mod 初始化失败!", e);
            throw new RuntimeException("[在线时间] Mod 初始化失败", e);
        }
        registerCommands();
    }

    public static ModConfig getConfig() {
        return config;
    }



    public static PlayerTimeTracker getTimeTracker() {
        return timeTracker;
    }


    public static void registerCommands() {
        CommandRegistrationCallback.EVENT.register((dispatcher, registryAccess, environment) -> {
            dispatcher.register(CommandManager.literal("onlineTime")
                    .requires(source -> source.hasPermissionLevel(0))
                    .executes(context -> showOnlineTime(context.getSource(), 1)) // 默认第一页
                    .then(CommandManager.argument("page", IntegerArgumentType.integer(1))
                            .executes(context -> showOnlineTime(
                                    context.getSource(),
                                    IntegerArgumentType.getInteger(context, "page")
                            ))
                    )
            );
        });
    }

    private static int showOnlineTime(ServerCommandSource source, int requestedPage) {
        ServerPlayerEntity player = source.getPlayer();
        if (player == null) return 0;

        CompletableFuture.runAsync(() -> {
            PlayerTimeTracker tracker = getTimeTracker();
            if (tracker != null) {
                Map<String, String> stats = tracker.getWhitelistedPlayerStats();

                List<String> sorted = stats.entrySet().stream()
                        .sorted((a, b) -> comparePlayTime(b.getValue(), a.getValue()))
                        .map(entry -> formatPlayerTime(entry.getKey(), entry.getValue()))
                        .collect(Collectors.toList());

                sendPaginatedMessage(player, sorted, requestedPage);
            }
        }, Util.getMainWorkerExecutor());

        return 1;
    }



    private static int comparePlayTime(String a, String b) {
        return Long.compare(parseTimeToSeconds(a), parseTimeToSeconds(b));
    }

    private static long parseTimeToSeconds(String timeStr) {
        long seconds = 0;
        String[] parts = timeStr.split(" ");
        for (String part : parts) {
            if (part.contains("h")) {
                seconds += Integer.parseInt(part.replace("h", "")) * 3600;
            } else if (part.contains("m")) {
                seconds += Integer.parseInt(part.replace("m", "")) * 60;
            }
        }
        return seconds;
    }

    private static String formatPlayerTime(String name, String timeStr) {
        return String.format("§e%s§r: %s", name, timeStr.split(" \\| ")[0]); // 只显示总时长
    }

    private static void sendPaginatedMessage(ServerPlayerEntity player, List<String> lines, int page) {
        int pageSize = 10;
        int totalPages = (lines.size() + pageSize - 1) / pageSize;
        page = Math.max(1, Math.min(page, totalPages));

        int from = (page - 1) * pageSize;
        int to = Math.min(from + pageSize, lines.size());

        // 标题
        player.sendMessage(Text.literal("§6===== 玩家在线时长 (第 " + page + "/" + totalPages + " 页) ====="), false);

        // 内容
        for (int i = from; i < to; i++) {
            player.sendMessage(Text.literal(lines.get(i)), false);
        }

        // 翻页按钮
        MutableText footer = Text.literal("");
        if (page > 1) {
            int finalPage = page;
            footer.append(Text.literal("§a[上一页]")
                    .styled(style -> style.withClickEvent(new ClickEvent(
                            ClickEvent.Action.RUN_COMMAND,
                            "/onlineTime " + (finalPage - 1)
                    )))
                    .append(" "));
        }

        footer.append(Text.literal("§7共 " + lines.size() + " 位玩家"));

        if (page < totalPages) {
            int finalPage1 = page;
            footer.append(" ").append(Text.literal("§a[下一页]")
                    .styled(style -> style.withClickEvent(new ClickEvent(
                            ClickEvent.Action.RUN_COMMAND,
                            "/onlineTime " + (finalPage1 + 1)
                    ))));
        }

        player.sendMessage(footer, false);
    }

}